package org.expeditee.gui;

import java.awt.Color;

import org.expeditee.items.Item;
import org.expeditee.items.Text;

public class FrameCreator {

	private String _name;
	
	private final static int START_Y = 40;

	private final static int START_X = 20;

	private Frame _current = null;

	private int _lastX = START_X;

	private int _maxX = 0;

	private int _lastY =  START_Y;

	private Text _Mnext = new Text(-1, "@Next");

	private Text _Mprev = new Text(-1, "@Previous");

	private Item _next;

	private Item _prev;

	public FrameCreator(String name, boolean recreate) {
		int num = -1;
		if (!recreate)
			num = FrameIO.getLastNumber(name);

		Frame toUse = null;

		if (num == -1)
			toUse = FrameIO.CreateFrameset(name, FrameIO.HELP_PATH);
		else
			toUse = FrameIO.CreateFrame(name, name, null);

		_name = name;
		_lastY += toUse.getTitle().getPosition().y;
		_current = toUse;

		// set positions of next\prev frame links
		_Mnext.setPosition(FrameGraphics.getMaxSize().width - 100,
				FrameGraphics.getMaxSize().height - 15);
		_Mprev.setPosition(50, FrameGraphics.getMaxSize().height - 15);
	}

	/**
	 * Creates the next frame in the frameset, with a previous button already
	 * added and linked to the last frame. _current then gets updated to point
	 * at the newly created Frame, and _lastY is reset
	 */
	public void createNextFrame() {
		Frame newFrame = FrameIO.CreateFrame(_current.getFramesetName(),
				_name, null);

		// add link to previous frame
		_prev = _Mprev.copy();
		_prev.setID(newFrame.getNextItemID());
		_prev.setLink(_current.getFrameName());
		newFrame.addItem(_prev);

		// add link to new frame
		_next = _Mnext.copy();
		_next.setID(_current.getNextItemID());
		_next.setLink(newFrame.getFrameName());
		_current.addItem(_next);
		FrameIO.SaveFrame(_current, false);

		// update\reset globals
		_current = newFrame;
		_lastY = START_Y + _current.getTitle().getPosition().y;;
		_lastX = START_X;
		_maxX = 0;
	}

	public void addItem(Item toAdd) {
		// if we have reached the end of the Y axis, try moving over on the X
		// axis
		if (_lastY >= _Mprev.getY() - _Mprev.getBoundsHeight()) {
			_lastX = _maxX + 10;
			_lastY = START_Y;
		}

		// if there is no more room on the X axis, we have to start a new frame
		if (toAdd.getBoundsWidth() + _lastX > FrameGraphics.getMaxSize().width) {
			createNextFrame();
		}

		toAdd.setPosition(_lastX, _lastY);
		toAdd.setOffset(0, 0);
		toAdd.setID(_current.getNextItemID());

		/*
		 * List<Item> items = new ArrayList<Item>(4);
		 * 
		 * items.add(_current.getName()); items.add(_current.getTitle());
		 * items.add(_prev); items.add(_next);
		 * 
		 * _current.getItems().removeAll(items);
		 * 
		 * //new system.agents.Format().process(_current);
		 * 
		 * //FrameUtils.Align(items, toAdd, true);
		 */
		_current.addItem(toAdd);
		// _current.addAllItems(items);

		FrameIO.SaveFrame(_current, false);

		_lastY = toAdd.getY() + toAdd.getBoundsHeight();
		_maxX = Math.max(toAdd.getX() + toAdd.getBoundsWidth(), _maxX);
	}

	public void addText(String toAdd) {
		/*
		 * Text text = _current.createNewText(); text.setText(toAdd);
		 * 
		 * addItem(text);
		 */
		addText(toAdd, null);
	}

	public void addText(String toAdd, Color c) {
		Text text = _current.createNewText();
		text.setText(toAdd);
		text.setColor(c);

		addItem(text);
	}

	/**
	 * Returns the Frame name of the current (latest) frame in the message
	 * frameset.
	 * 
	 * @return The Frame currently being used in the message frameset
	 */
	public String getCurrent() {
		if (_current == null)
			return null;

		return _current.getFrameName();
	}
}
