package org.expeditee.io;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;

import org.expeditee.gui.Frame;
import org.expeditee.items.Constraint;
import org.expeditee.items.Dot;
import org.expeditee.items.Item;
import org.expeditee.items.Line;
import org.expeditee.items.Picture;
import org.expeditee.items.Text;
import org.expeditee.stats.SessionStats;

/**
 * Writes a Frame out to a KMS format file.
 * 
 * @author jdm18
 * 
 */
public class KMSWriter extends DefaultFrameWriter {

	private ProxyWriter _writer = null;

	private StringBuilder _stringWriter = null;

	// keep track of methods that are put on the same line
	private static LinkedHashMap<String, Method> _ItemTags = null;

	private static LinkedHashMap<String, Method> _FrameTags = null;

	private static LinkedHashMap<String, Method> _BodyTags = null;

	private static final char TERMINATOR = 'Z';

	private static final char DELIMITER = '+';

	public KMSWriter() {
		if (_ItemTags != null && _FrameTags != null)
			return;

		_ItemTags = new LinkedHashMap<String, Method>();
		_FrameTags = new LinkedHashMap<String, Method>();
		_BodyTags = new LinkedHashMap<String, Method>();

		Class[] param = {};

		try {
			_FrameTags.put("A", Frame.class.getMethod("getFrameName", param));
			_FrameTags.put("V", Frame.class.getMethod("getVersion", param));
			_FrameTags.put("v", Frame.class
					.getMethod("getFormatVersion", param));
			_FrameTags.put("p", Frame.class.getMethod("getProtection", param));
			_FrameTags.put("U", Frame.class.getMethod("getOwner", param));
			_FrameTags.put("D", Frame.class.getMethod("getDateCreated", param));
			_FrameTags.put("M", Frame.class.getMethod("getLastModifyUser",
					param));
			_FrameTags.put("d", Frame.class.getMethod("getLastModifyDate",
					param));
			_FrameTags.put("F", Frame.class.getMethod("getFrozenDate", param));

			_BodyTags.put("O", Frame.class.getMethod("getForegroundColor",
					param));
			_BodyTags.put("B", Frame.class.getMethod("getBackgroundColor",
					param));
			_BodyTags
					.put("v", Frame.class.getMethod("getFormatVersion", param));

			_ItemTags.put("S", Item.class.getMethod("getID", param));
			_ItemTags.put("s", Item.class.getMethod("getDateCreated", param));
			_ItemTags.put("d", Item.class.getMethod("getColor", param));
			_ItemTags.put("G", Item.class
					.getMethod("getBackgroundColor", param));
			_ItemTags.put("P", Item.class.getMethod("getPosition", param));
			_ItemTags.put("F", Item.class.getMethod("getLink", param));
			_ItemTags.put("X", Item.class.getMethod("getAction", param));
			_ItemTags.put("x", Item.class.getMethod("getActionMark", param));
			_ItemTags.put("U", Item.class.getMethod("getActionCursorEnter",
					param));
			_ItemTags.put("V", Item.class.getMethod("getActionCursorLeave",
					param));
			_ItemTags.put("W", Item.class.getMethod("getActionEnterFrame",
					param));
			_ItemTags.put("Y", Item.class.getMethod("getActionLeaveFrame",
					param));
			_ItemTags.put("D", Item.class.getMethod("getData", param));
			_ItemTags.put("u", Item.class.getMethod("getHighlight", param));
			_ItemTags.put("e", Item.class.getMethod("getFillColor", param));
			_ItemTags.put("i", Item.class.getMethod("getFillPattern", param));
			_ItemTags.put("o", Item.class.getMethod("getOwner", param));
			_ItemTags.put("n", Item.class.getMethod("getLinkMark", param));
			_ItemTags.put("q", Item.class.getMethod("getLinkFrameset", param));
			_ItemTags.put("y", Item.class.getMethod("getLinkTemplate", param));
			_ItemTags.put("g", Item.class.getMethod("getLinePattern", param));

			_ItemTags.put("j", Item.class.getMethod("getArrow", param));

			_ItemTags.put("f", Text.class.getMethod("getFont", param));
			_ItemTags.put("t", Text.class.getMethod("getSpacing", param));
			_ItemTags.put("T", Text.class.getMethod("getText", param));

			_ItemTags.put("a", Text.class.getMethod("getWordSpacing", param));
			_ItemTags.put("b", Text.class.getMethod("getLetterSpacing", param));
			_ItemTags
					.put("m", Text.class.getMethod("getInitialSpacing", param));
			_ItemTags.put("w", Text.class.getMethod("getWidth", param));
			_ItemTags.put("k", Text.class.getMethod("getJustification", param));

			_ItemTags.put("h", Dot.class.getMethod("getThickness", param));
			_ItemTags.put("l", Dot.class.getMethod("getLineIDs", param));
			_ItemTags.put("c", Dot.class.getMethod("getConstraintIDs", param));

			// Lines and constraints are created differently
			/*
			 * _LineTags.put("L", Line.class.getMethod("getID", param));
			 * _LineTags.put("s", Line.class.getMethod("getLineEnds", param));
			 * _LineTags.put("g", Line.class.getMethod("getLinePattern",
			 * param));
			 * 
			 * _ConstraintTags.put("C", Constraint.class.getMethod("getID",
			 * (Class[]) null)); _ConstraintTags.put("s",
			 * Constraint.class.getMethod("getLineEnds", (Class[]) null));
			 */
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void initialise(Frame start) throws IOException {
		String name = start.getFramesetName().toLowerCase();

		if (_filename == null)
			_filename = start.path + name + File.separator + name + "."
					+ start.getFrameNumber();

		_stringWriter = new StringBuilder();
		if (_filename.toLowerCase().equals("clipboard")) {
			_writer = new ProxyWriter(true);
			_filename = "Clipboard";
		} else
			_writer = new ProxyWriter(new FileWriter(_filename));

		// _writer = new BufferedWriter(new FileWriter(filepath.toLowerCase() +
		// filename.toLowerCase()));
	}

	/**
	 * Writes the given Frame (and all items it contains) to a KMS file. Note:
	 * File path and location must be set before calling this or it will do
	 * nothing.
	 * 
	 * @param frame
	 *            The Frame to write out to the file.
	 * @throws IOException
	 *             Any exceptions occured by the BufferedWriter.
	 */
	public void outputFrame(Frame frame) throws IOException {
		if (_writer == null)
			return;

		writeHeader(frame);
		// writeBody(frame);

		// write out each Item in the Frame.
		List<Item> items = frame.getItems();

		// do not write items with ID -1, also skip any lines
		for (Item i : items) {
			if (i.getID() >= 0 && !(i instanceof Line))
				writeItem(i);
		}

		// write any lines or constraints
		writeLineData();
		writeConstraintData();

		return;
	}

	private void writeHeader(Frame toWrite) throws IOException {
		Iterator<String> it = _FrameTags.keySet().iterator();
		Object[] param = {};

		while (it.hasNext()) {
			String tag = it.next();
			try {
				Object o = _FrameTags.get(tag).invoke(toWrite, param);
				o = Conversion.ConvertToKMS(_FrameTags.get(tag), o);
				if (o != null) {
					writeLine(tag, (String) o);
				}
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		writeTerminator();
		it = _BodyTags.keySet().iterator();

		while (it.hasNext()) {
			String tag = it.next();
			try {
				Object o = _BodyTags.get(tag).invoke(toWrite, param);
				o = Conversion.ConvertToKMS(_BodyTags.get(tag), o);
				if (o != null) {
					writeLine(tag, (String) o);
				}
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void writeLine(String tag, String line) throws IOException {
		writeLine(DELIMITER + tag + DELIMITER + " " + line);
	}

	private void writeTerminator() throws IOException {
		writeLine("" + DELIMITER + TERMINATOR + DELIMITER);
	}

	// writes the given line out to the file
	private void writeLine(String line) throws IOException {
		// do not write empty lines
		if (line == null)
			return;

		String toWrite = line + "\n";

		_writer.write(toWrite);
		_stringWriter.append(toWrite);
	}

	// writes the given Item out to the file
	public void writeItem(Item item) throws IOException {
		if (_writer == null)
			return;

		writeLine("");
		// Pictures are saved as the corresponding text items that created them
		if (item instanceof Picture)
			item = ((Picture) item).getText();

		/*
		 * if(item instanceof Text) writeClass(item); else if(item instanceof
		 * Dot) writeDot((Dot) item);
		 */
		if (item.getLines().size() > 0)
			writePoint(item);
		else if (!(item instanceof Line))
			writeClass(item);
		// lines are saved at the end of the file
		/*
		 * else if (item instanceof Line) ;
		 */
		else
			System.out.println("Unknown Item: " + item.getID());
	}

	private List<Item> _points = new LinkedList<Item>();

	// Writes out a Dot to the file
	protected void writePoint(Item point) throws IOException {
		_points.add(point);
		writeClass(point);
	}

	// writes out all lines to the file
	private void writeLineData() throws IOException {
		List<Line> seen = new LinkedList<Line>();

		// loop through all points stored
		for (int i = 0; i < _points.size(); i++) {
			List<Line> lines = _points.get(i).getLines();

			// if this point is part of one or more lines
			if (lines != null && lines.size() > 0) {
				for (Line line : lines) {

					// only output new lines that have not yet been output
					if (!seen.contains(line)) {
						writeLine("L", line.getID() + " " + line.getLineType());
						writeLine("s", line.getLineEnds());

						// add this line to the list of lines that have been
						// seen
						seen.add(line);
					}

				}
			}
		}
	}

	// writes out any constraints to the file
	private void writeConstraintData() throws IOException {
		// outputs any constraints the points have

		// loop through all the points
		while (_points.size() > 0) {
			Item i = _points.get(0);

			if (i instanceof Dot) {
				Dot p = (Dot) i;

				// if there are any constraints to write
				if (p.getConstraints() != null) {
					List<Constraint> constraints = p.getConstraints();

					// do not write constraints that have already been
					// written
					for (Constraint c : constraints) {
						if (_points.contains(c.getStart())
								&& _points.contains(c.getEnd())) {
							writeLine("C", c.getID() + " "
									+ c.getConstraintType());
							writeLine("s", c.getLineEnds());
						}

					}
				}
			}
			// remove the point from the list
			_points.remove(0);

		}
	}

	@Override
	protected String finaliseFrame() throws IOException {
		writeLine("" + DELIMITER + TERMINATOR + DELIMITER);

		writeLine(SessionStats.getFrameEventList());

		_writer.flush();
		_writer.close();
		_writer = null;

		return "Frame successfully written to " + _filename;
	}

	private void writeClass(Object toWrite) throws IOException {
		Iterator<String> it = _ItemTags.keySet().iterator();
		Object[] param = {};

		while (it.hasNext()) {
			String tag = it.next();
			Method toRun = _ItemTags.get(tag);
			Class declarer = toRun.getDeclaringClass();
			if (declarer == toWrite.getClass()
					|| declarer == toWrite.getClass().getSuperclass()
					|| declarer == toWrite.getClass().getSuperclass()
							.getSuperclass()) {
				try {
					Object o = toRun.invoke(toWrite, param);
					o = Conversion.ConvertToKMS(toRun, o);
					if (o != null) {
						if (o instanceof List) {
							for (Object line : (List) o)
								writeLine(tag, (String) line);
						} else
							writeLine(tag, (String) o);
					}
				} catch (IllegalArgumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		/*
		 * Method[] methods = toWrite.getClass().getMethods(); ArrayList<String>
		 * meths = new ArrayList<String>();
		 * 
		 * for(Method m : methods) meths.add(m.getName());
		 * 
		 * String start = _Tags.get("DelimiterStart"); String end =
		 * _Tags.get("DelimiterEnd");
		 * 
		 * Iterator<String> it = _Tags.keySet().iterator();
		 * 
		 * String term = null; LinkedList<Method> seen = new LinkedList<Method>();
		 * 
		 * while(it.hasNext()){ String name = it.next(); String cName =
		 * className;
		 * 
		 * if(writeSuper && name.startsWith(superName)) cName = superName;
		 * 
		 * if(name.startsWith(cName)){ String tag = start + _Tags.get(name) +
		 * end; String output = "";
		 * 
		 * //check for other methods that go on this line LinkedList<String>
		 * shared = _SharedLine.get(name); LinkedList<String> runMethods = new
		 * LinkedList<String>();
		 * 
		 * if(shared != null) runMethods.addAll(shared);
		 * 
		 * name = name.substring((cName + ".").length());
		 * runMethods.addFirst(name);
		 * 
		 * //loop through all methods and concat their output Iterator
		 * methodIterator = runMethods.iterator();
		 * 
		 * while(methodIterator.hasNext()){ name = "get" +
		 * methodIterator.next();
		 * 
		 * int ind = meths.indexOf(name);
		 * 
		 * if(ind >= 0){ Method m = methods[ind];
		 * 
		 * if(!seen.contains(m)){ try { if(m.getParameterTypes().length == 0){
		 * Object val = m.invoke(toWrite, (Object[]) null);
		 * 
		 * if(val instanceof List){ int count = 0; for(Object o : (List) val){
		 * if(count == 0) output += " " +
		 * KMSConversion.ConvertToKMS(name.substring(3), o); else output += "\n" +
		 * tag + " " + KMSConversion.ConvertToKMS(name.substring(3), o);
		 * 
		 * count++; } }else{ String res =
		 * KMSConversion.ConvertToKMS(name.substring(3), val);
		 * 
		 * if(res != null) output += " " + res; } } } catch (Exception e) {
		 * Logger.Log(e); }
		 * 
		 * seen.add(m); } }else if(name.equals("getTerminator")){ term = start +
		 * _Tags.get(cName + ".Terminator") + end; }else System.out.println("No
		 * method found: " + cName + "." + name); }
		 * 
		 * if(output.length() > 0) writeLine(tag + output); } } if(term != null)
		 * writeLine(term);
		 */
	}

	/*
	 * private void writeBody(Object toWrite) throws IOException{
	 * writeClass(toWrite, "Body", " ", false); }
	 */

	/**
	 * Gets a string representation of the frame file contents.
	 */
	public String getFileContents() {
		return _stringWriter.toString();
	}
}
