package org.expeditee.simple;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.gui.FrameIO;

public class Context {
	private Primitives primitives_ = new Primitives();

	private Pointers pointers_ = new Pointers();

	public Primitives getPrimitives() {
		return primitives_;
	}

	public Pointers getPointers() {
		return pointers_;
	}

	public Context() {
	}

	public int size() {
		return primitives_.size() + pointers_.size();
	}

	public void clear() {
		primitives_.clear();
		pointers_.clear();
	}

	public void display() {
		if (size() == 0) {
			FrameGraphics.ErrorMessage("No variables exist!");
			return;
		}

		primitives_.display();
		pointers_.display();
	}

	/**
	 * 
	 * @param var1
	 * @param var2
	 * @return
	 * @throws Exception
	 *             if the variables could not be converted to the same type.
	 */
	public boolean equalValues(String var1, String var2) throws Exception {
		if (Primitives.isPrimitive(var1) && Primitives.isPrimitive(var2)) {
			return primitives_.equalValues(var1, var2);
		} else if (Pointers.isPointer(var1) && Pointers.isPointer(var2)) {
			throw new UnsupportedOperationException(
					"Pointer comparison not yet implemented");
		}
		return false;
	}

	public void readFrame(String frameNameVar, String frameVar,
			String successVar) throws Exception {
		// Get the values to be set
		String frameName = primitives_.getStringValue(frameNameVar);
		Frame currentFrame = FrameIO.LoadFrame(frameName);

		pointers_.setObject(frameVar, currentFrame);

		if (successVar != null) {
			Boolean success = currentFrame != null;
			primitives_.setValue(successVar, new SBoolean(success));
		}
	}

	public void createFrame(String framesetNameVar, String frameVar,
			String successVar) throws Exception {
		// Get the values to be set
		String framesetName = primitives_.getStringValue(framesetNameVar);
		Frame currentFrame = FrameIO.CreateFrame(framesetName, "", null);
		pointers_.setObject(frameVar, currentFrame);

		if (successVar != null) {
			Boolean success = currentFrame != null;
			primitives_.setValue(successVar, new SBoolean(success));
		}
	}

	public void closeFrame(String frameVar, String successVar) throws Exception {
		// Get the values to be set
		Frame frame = (Frame) pointers_.getVariable(frameVar).getValue();
		String contents = FrameIO.SaveFrame(frame, false);

		if (successVar != null) {
			Boolean success = contents != null;
			primitives_.setValue(successVar, new SBoolean(success));
		}
	}

	public void closeWriteFile(String fileVar) throws Exception {
		BufferedWriter bw = (BufferedWriter) pointers_.getVariable(fileVar)
				.getValue();
		bw.close();
	}

	public void writeFile(String fileVar, String text) throws Exception {
		BufferedWriter bw = (BufferedWriter) pointers_.getVariable(fileVar)
				.getValue();
		bw.write(text);
	}

	public void openWriteFile(String fileNameVar, String fileVar)
			throws Exception {
		openWriteFile(fileNameVar, fileVar, null);
	}

	public void openWriteFile(String fileNameVar, String fileVar,
			String successVar) throws Exception {
		// Get the values to be set
		File filePath = new File(primitives_.getStringValue(fileNameVar));
		BufferedWriter currentFile = new BufferedWriter(
				new FileWriter(filePath));

		pointers_.setObject(fileVar, currentFile);

		if (successVar != null) {
			Boolean success = currentFile != null;
			primitives_.setValue(successVar, new SBoolean(success));
		}
	}

	public boolean isDefined(String varName) {
		try {
			if (Primitives.isPrimitive(varName)) {
				getPrimitives().getVariable(varName);
			} else {
				getPointers().getVariable(varName);
			}
			return true;
		} catch (Exception e) {
			return false;
		}
	}
}